import cliutils, docopt, logging, jester, json, os, strutils, tables

import personal_measure_apipkg/configuration
import personal_measure_apipkg/version
import personal_measure_apipkg/api

const DEFAULT_CONFIG = PMApiConfig(
  authSecret: "change me",
  dbConnString: "",
  debug: false,
  port: 8080,
  pwdCost: 11)

proc loadConfig*(args: Table[string, docopt.Value] = initTable[string, docopt.Value]()): PMApiConfig  =

  let filePath =
      if args["--config"]: $args["--config"]
      else: "personal_measure.config.json"

  var json: JsonNode
  try: json = parseFile(filePath)
  except:
    json = %DEFAULT_CONFIG
    if not existsFile(filePath):
      info "created new configuration file \"" & filePath & "\""
      filePath.writeFile($json)
    else:
      warn "Cannot read configuration file \"" & filePath & "\":\n\t" &
        getCurrentExceptionMsg()

  let cfg = CombinedConfig(docopt: args, json: json)

  result = PMApiConfig(
    authSecret: cfg.getVal("authSecret"),
    dbConnString: cfg.getVal("dbConnString"),
    debug: "true".startsWith(cfg.getVal("debug", "false").toLower()),
    port: parseInt(cfg.getVal("port", "8080")),
    pwdCost: parseInt(cfg.getVal("pwdCost", "11")))
    
proc initContext(args: Table[string, docopt.Value]): PMApiContext =

  var cfg: PMApiConfig
  var db: PMApiDb

  try: cfg = loadConfig(args)
  except: raiseEx "Unable to load configuration: \n\t" & getCurrentExceptionMsg()

  try: db = connect(cfg.dbConnString)
  except: raiseEx "Unable to connect to the database:\n\t" & getCurrentExceptionMsg()

  result = PMApiContext(cfg: cfg, db: db)

when isMainModule:

  try:
    let doc = """
Usage:
  personal_measure_api test [options]
  personal_measure_api serve [options]

Options:

  -C, --config <cfgFile>    Location of the config file (defaults to personal_measure.config.json)
"""
    logging.addHandler(newConsoleLogger())

    # Initialize our service context
    let args = docopt(doc, version = PM_API_VERSION)
    let ctx = initContext(args)

    if args["test"]:
      echo "Test"

    if args["serve"]:
      start(PMApiConfig(
        debug: true,
        port: 8090,
        pwdCost: 11,
        dbConnString: "host=localhost port=5500 username=postgres password=password dbname=personal_measure"))
  except:
    fatal "pit: " & getCurrentExceptionMsg()
    #raise getCurrentException()
    quit(QuitFailure)
